#include <string.h>
#include <stdlib.h>
#include "ostdb.h"
#include "cdbslib.h"
#include "config.h"
#include "loadfile.h"
#include "dbquery.h"

#define  SZ_TEXT	16000

/* COMPFILE_QRY -- Query database for comparison file
 *
 * i: inst	instrument name
 * i: reftype	reference file type
 * i: usedate	useafter date
 * i: obsmode	observation mode clause
 * i: reffile	reference file for which comparison file is sought
 * o: compfile	comparison file name
 * o: useafter	useafter date
 *
 * History:
 * 10/14/09 53705 MSwam   only return 1 comparison file (use latest,greatest)
 * 01/27/10 64387 MSwam   compiling under GCC 4.1.1 causes strcmp against NULL
 *                          to crash- substituted blank instead
 */

void compfile_qry (char *inst, char *reftype, char *usedate, 
		   char *obsmode, char *compfile, char *useafter, int maxch)
{
    static char *nomode_compfile_qry = 
"select distinct a.file_name, a.useafter_date from ^_file a \
where a.reference_file_type = ^ and a.reject_flag = 'N' and \
a.file_name in (select a.file_name from ^_file a where \
a.reference_file_type = ^ and \
a.useafter_date = (select max(a.useafter_date) \
from ^_file a where a.reference_file_type = ^ and \
a.reject_flag = 'N' and a.useafter_date <= ^) )";

    static char *mode_compfile_qry = 
"select distinct a.file_name, a.useafter_date from ^_file a, \
^_row b where a.reference_file_type = ^ and \
a.file_name = b.file_name and a.expansion_number = \
b.expansion_number and a.reject_flag = 'N' and ^ and \
a.file_name in (select a.file_name from ^_file a where \
a.reference_file_type = ^ and \
a.useafter_date = (select max(a.useafter_date) \
from ^_file a, ^_row b where a.reference_file_type = ^ and \
a.file_name = b.file_name and a.expansion_number = \
b.expansion_number and a.reject_flag = 'N' and a.useafter_date <= \
^ and ^) )";

    context 	*qry;
    textbuf	filebuf, datebuf;
    int		nfile, ifile, isfits, copy;
    char	*type, *cutoff, *fileptr, *dateptr, *lastfile;
    char	fname[SZ_FNAME+1], date[SZ_STRING+1], blank[2]=" ";

    /* Initialize query and bind output variables */

    qry = init_query ();
    ostdb_query_setup (qry->connection, "compname_qry", 1, STDB_SQL, 
		       qry->server, qry->cdbs_db, NULL);

    stdb_bind_string (qry->connection, "file_name", fname, SZ_FNAME);
    stdb_bind_string (qry->connection, "useafter_date", date, SZ_STRING);

    /* Create quoted versions of input strings */

    cutoff = strclone (enquote (usedate));
    type = strclone (strucase (enquote (reftype)));

    /* Send query after filling in variables */

    if (obsmode[0] == EOS) {
	build_qry (qry->connection, nomode_compfile_qry, inst, 
		   type, inst, type, inst, type, cutoff);
    } else {
	build_qry (qry->connection, mode_compfile_qry, inst, inst, type, 
		   obsmode, inst, type, inst, inst, type, cutoff, obsmode);
    }

    /* Execute query */

    nfile = 0;
    isfits = -1;
    init_text (&filebuf, 10, 200);
    init_text (&datebuf, 10, 200);

    while (ostdb_get_exec (qry->connection) == STDB_CONTINUE) {
        strlcase (fname);
        strclean (fname);
	strclean (date);

        add_text (&filebuf, fname);
	add_text (&datebuf, date);

	/* Determine if any of the comparison files is a fits file */

	if (is_fitsname (fname)) {
	    if (isfits == -1) {
	        isfits = nfile;
	    } else {
	        isfits = -2; /* more than one fits file */
	    }
	}

	nfile ++;
    }

    stdb_close_query (qry->connection);
    free_query (qry);

    free (cutoff); cutoff=NULL;
    free (type); type=NULL;

    if (isfits >= 0) {
        /*  Only return the fits file */

        safecopy (compfile, get_text (&filebuf, isfits), maxch);
	safecopy (date, get_text (&datebuf, isfits), maxch); 

    } else {
        /* Either return all the fits files (if any)
	 * or all the files
	 */

        compfile[0] = EOS;
	useafter[0] = EOS;

        /* PR 53705 - only save latest (greatest) name, not more than 1 */
        lastfile = blank;
	for (ifile = 0; ifile < nfile; ifile ++) {
	    fileptr = get_text (&filebuf, ifile);
	    dateptr = get_text (&datebuf, ifile);

	    if (isfits == -2) {
	        copy = is_fitsname (fileptr);
	    } else {
	        copy = YES;
	    }

	    if (copy) {
              if (strcmp(fileptr, lastfile) > 0) {
                lastfile = fileptr;
	        if (build_list (fileptr, compfile, maxch) == ERR)
	            break;

	        if (build_list (dateptr, useafter, maxch) == ERR)
	            break;
              }               
	    }
	}
    }
}

